@@ -21,7 +21,6 @@ from ..utils.general import Gif_Maker
 from datetime import datetime
 from ..utils.general.Wrapit import Wrapit
 from hamcrest import assert_that, equal_to, none, not_none, contains_string
-
 from pytest_html import extras
 
 
@@ -209,12 +208,12 @@ class Base_Page(Borg, unittest.TestCase):
     #         self.__class__ = pp_PageFactory.PageFactory.get_page_object(page_name, base_url=self.base_url).__class__
 
     def register_driver(self, remote_flag, os_name, os_version, browser, browser_version, remote_project_name,
-                        remote_build_name):
+                        remote_build_name, wdm):
         "Register the driver with Page."
         self.set_screenshot_dir(os_name, os_version, browser, browser_version)  # Create screenshot directory
         self.set_log_file()
         self.driver = self.driver_obj.get_web_driver(remote_flag, os_name, os_version, browser, browser_version,
-                                                     remote_project_name, remote_build_name)
+                                                     remote_project_name, remote_build_name, wdm)
         self.driver.implicitly_wait(5)
         self.driver.maximize_window()
 
@@ -273,7 +272,7 @@ class Base_Page(Borg, unittest.TestCase):
         return self.calling_module
 
     def set_directory_structure(self):
-        "Setup the required directory structure if it is not already present"
+        "Set up the required directory structure if it is not already present"
         try:
             self.screenshots_parent_dir = os.path.abspath(
                 os.path.join(os.curdir, 'screenshots', self.product))  # Real screencap folder dir
@@ -551,13 +550,12 @@ class Base_Page(Borg, unittest.TestCase):
         # if (hasattr(element, attribute_name)):
         #     attribute_value = element.get_attribute(attribute_name)
         attribute_value = element.get_attribute(attribute_name)
-
         return attribute_value
 
     def check_alert(self):
         result_flag = False
         try:
-            self.driver.switch_to.alert
+            alert = self.driver.switch_to.alert
             result_flag = True
         except:
             pass
@@ -569,8 +567,10 @@ class Base_Page(Borg, unittest.TestCase):
             msg = alert.text
             alert.accept()
             self.write('Close Alert box with message: ' + msg)
+            return True
         except:
             self.write('Switch and close alert box error')
+            return False
 
     def highlight_element(self, element, wait_seconds=0):
         "Highlights a Selenium webdriver element"
@@ -606,7 +606,6 @@ class Base_Page(Borg, unittest.TestCase):
                 self.write("Check your locator-'%s,%s' in the conf/locators.conf file" % (locator[0], locator[1]))
             self.exceptions.append(
                 "Check your locator-'%s,%s' in the conf/locators.conf file" % (locator[0], locator[1]))
-
         return dom_element
 
     def get_dropdown_element(self, locator, verbose_flag=True):
@@ -635,7 +634,7 @@ class Base_Page(Borg, unittest.TestCase):
             self.write(str(e), 'debug')
             self.write("Error while parsing locator")
             self.exceptions.append(
-                "Unable to split the locator-'%s' in the conf/locators.conf file" % (locator[0], locator[1]))
+                "Unable to split the locator-'%s,%s' in the conf/locators.conf file" % (locator[0], locator[1]))
 
         return result
 
@@ -648,7 +647,7 @@ class Base_Page(Borg, unittest.TestCase):
             if self.highlight_flag is True:
                 self.highlight_elements(dom_elements)
         except Exception as e:
-            if msg_flag == True:
+            if msg_flag:
                 self.write(str(e), 'debug')
                 self.write("Check your locator-'%s,%s' in the conf/locators.conf file" % (locator[0], locator[1]))
             self.exceptions.append(
@@ -665,13 +664,12 @@ class Base_Page(Borg, unittest.TestCase):
             if self.highlight_flag is True:
                 self.highlight_elements(dom_elements)
         except Exception as e:
-            if msg_flag == True:
+            if msg_flag:
                 self.write(str(e), 'debug')
                 self.write("Check your tag -'%s' " % tag)
             self.exceptions.append(
                 "Unable to locate the dom_elements with tag %s" % (
                     tag))
-
         return dom_elements
 
     def get_elements_under_element_by_tag(self, element, tag, msg_flag=True):
@@ -682,13 +680,12 @@ class Base_Page(Borg, unittest.TestCase):
             if self.highlight_flag is True:
                 self.highlight_elements(dom_elements)
         except Exception as e:
-            if msg_flag == True:
+            if msg_flag:
                 self.write(str(e), 'debug')
                 self.write("Check your tag -'%s' " % tag)
             self.exceptions.append(
                 "Unable to locate the dom_elements under element %s with tag %s" % (
                     element, tag))
-
         return dom_elements
 
     def click_element(self, locator, wait_time=1):
@@ -705,11 +702,10 @@ class Base_Page(Borg, unittest.TestCase):
             self.write('Exception when clicking link with path: %s' % locator)
             self.exceptions.append(
                 "Error when clicking the dom_element with path,'%s' in the conf/locators.conf file" % locator)
-
         return result_flag
 
     def click_dom_element(self, dom_element, wait_time=1):
-        "Click the button supplied, the dom_element should be get by get_elements"
+        "Click the button supplied, the dom_element should be got by get_elements"
         result_flag = False
         try:
             link = dom_element
@@ -722,27 +718,44 @@ class Base_Page(Borg, unittest.TestCase):
             self.write('Exception when clicking link with path: %s' % dom_element)
             self.exceptions.append(
                 "Error when clicking the dom_element with path,'%s' in the conf/locators.conf file" % dom_element)
-
         return result_flag
 
     def double_click_element(self, locator, wait_time=1):
-        "Double click the element"
+        "Double-click the element"
         # Note: perform() of ActionChains does not return a bool
         # So we have no way of returning a bool when hover is called
-        element = self.get_element(locator)
-        action_obj = ActionChains(self.driver)
-        action_obj.double_click(element)
-        action_obj.perform()
-        self.wait(wait_time)
+        result_flag = False
+        try:
+            element = self.get_element(locator)
+            action_obj = ActionChains(self.driver)
+            action_obj.double_click(element)
+            action_obj.perform()
+            self.wait(wait_time)
+            result_flag = True
+        except Exception as e:
+            self.write(str(e), 'debug')
+            self.write('Exception when clicking link with path: %s' % locator)
+            self.exceptions.append(
+                "Error when double clicking the element with path,'%s' in the conf/locators.conf file" % locator)
+        return result_flag
 
     def double_click_dom_element(self, element, wait_time=1):
-        "Double click the element"
+        "Double-click the element"
         # Note: perform() of ActionChains does not return a bool
         # So we have no way of returning a bool when hover is called
-        action_obj = ActionChains(self.driver)
-        action_obj.double_click(element)
-        action_obj.perform()
-        self.wait(wait_time)
+        result_flag = False
+        try:
+            action_obj = ActionChains(self.driver)
+            action_obj.double_click(element)
+            action_obj.perform()
+            self.wait(wait_time)
+            result_flag = True
+        except Exception as e:
+            self.write(str(e), 'debug')
+            self.write('Exception when clicking link with path: %s' % element)
+            self.exceptions.append(
+                "Error when double clicking the dom_element with path,'%s' in the conf/locators.conf file" % element)
+        return result_flag
 
     def set_text(self, locator, value, clear_flag=True):
         "Set the value of the text field"
@@ -768,7 +781,6 @@ class Base_Page(Borg, unittest.TestCase):
                 self.write('Could not write to text field: %s' % locator, 'debug')
                 self.write(str(e), 'debug')
                 self.exceptions.append("Could not write to text field- '%s' in the conf/locators.conf file" % locator)
-
         return result_flag
 
     def set_dom_text(self, dom_element, value, clear_flag=True):
@@ -815,7 +827,6 @@ class Base_Page(Borg, unittest.TestCase):
             self.exceptions.append(
                 # "Error when getting text from the DOM dom_element-'%s' in the conf/locators.conf file" % locator)
                 "Error when getting text from the DOM dom_element-'%s' in the conf/locators.conf file" % dom_element)
-
         return text
 
     def select_checkbox(self, locator):
@@ -830,7 +841,6 @@ class Base_Page(Borg, unittest.TestCase):
         except Exception as e:
             self.write(e)
             self.exceptions.append("Error when selecting checkbox-'%s' in the conf/locators.conf file" % locator)
-
         return result_flag
 
     def select_dom_checkbox(self, element):
@@ -844,7 +854,6 @@ class Base_Page(Borg, unittest.TestCase):
         except Exception as e:
             self.write(e)
             self.exceptions.append("Error when selecting checkbox-'%s' by dom element" % element)
-
         return result_flag
 
     def deselect_checkbox(self, locator):
@@ -859,7 +868,6 @@ class Base_Page(Borg, unittest.TestCase):
         except Exception as e:
             self.write(e)
             self.exceptions.append("Error when deselecting checkbox-'%s' in the conf/locators.conf file" % locator)
-
         return result_flag
 
     def deselect_dom_checkbox(self, element):
@@ -873,7 +881,6 @@ class Base_Page(Borg, unittest.TestCase):
         except Exception as e:
             self.write(e)
             self.exceptions.append("Error when deselecting checkbox-'%s' by dom element" % element)
-
         return result_flag
 
     unselect_checkbox = deselect_checkbox  # alias the method
@@ -934,7 +941,6 @@ class Base_Page(Borg, unittest.TestCase):
         except Exception as e:
             self.write(e)
             self.exceptions.append("Error when selecting option from the drop-down '%s' " % locator)
-
         return result_flag
 
     def select_dropdown_option_by_textContent(self, locator, option_text):
@@ -952,7 +958,6 @@ class Base_Page(Borg, unittest.TestCase):
         except Exception as e:
             self.write(e)
             self.exceptions.append("Error when selecting option from the drop-down '%s' " % locator)
-
         return result_flag
 
     def select_dropdown_option_by_value(self, locator, value):
@@ -968,7 +973,6 @@ class Base_Page(Borg, unittest.TestCase):
         except Exception as e:
             self.write(e)
             self.exceptions.append("Error when selecting option from the drop-down '%s' " % locator)
-
         return result_flag
 
     def select_dropdownelement_option_by_text(self, element, option_text):
@@ -980,7 +984,6 @@ class Base_Page(Borg, unittest.TestCase):
             result_flag = False
             self.write(e)
             self.exceptions.append("Unable to find %s in dom dropdown %s by visible text" % (option_text, element))
-
         return result_flag
 
     def elementIsDisplayed(self, element):
@@ -992,7 +995,6 @@ class Base_Page(Borg, unittest.TestCase):
             result_flag = False
             self.write(e)
             self.exceptions.append("Element is not displayed" % (element))
-
         return result_flag
 
     def select_dropdownelement_option_by_value(self, element, option_value):
@@ -1004,7 +1006,6 @@ class Base_Page(Borg, unittest.TestCase):
             result_flag = False
             self.write(e)
             self.exceptions.append("Unable to find %s in dom dropdown %s by value" % (option_value, element))
-
         return result_flag
 
     def check_element_present(self, locator):
@@ -1012,7 +1013,6 @@ class Base_Page(Borg, unittest.TestCase):
         result_flag = False
         if self.get_element(locator, verbose_flag=False) is not None:
             result_flag = True
-
         return result_flag
 
     def check_element_displayed(self, locator):
@@ -1027,7 +1027,6 @@ class Base_Page(Borg, unittest.TestCase):
             self.write(e)
             self.exceptions.append(
                 "Web element not present in the page, please check the locator is correct -'%s' in the conf/locators.conf file" % locator)
-
         return result_flag
 
     def check_element_enabled(self, locator):
@@ -1042,7 +1041,6 @@ class Base_Page(Borg, unittest.TestCase):
             self.write(e)
             self.exceptions.append(
                 "Web element not present in the page, please check the locator is correct -'%s' in the conf/locators.conf file" % locator)
-
         return result_flag
 
     def check_dom_enabled(self, element):
@@ -1055,7 +1053,6 @@ class Base_Page(Borg, unittest.TestCase):
             self.write(e)
             self.exceptions.append(
                 "Web element not present in the page, please check the element is correct %s" % element)
-
         return result_flag
 
     def hit_enter(self, locator, wait_time=0.5):
@@ -1075,10 +1072,11 @@ class Base_Page(Borg, unittest.TestCase):
             element = self.get_element(locator)
             element.send_keys(Keys.ARROW_DOWN)
             self.wait(wait_time)
+            return True
         except Exception as e:
             self.write(str(e), 'debug')
             self.exceptions.append("An exception occurred when hitting enter")
-            return None
+            return False
 
     def scroll_down(self, locator, wait_time=0.5):
         "Scroll down"
@@ -1086,10 +1084,11 @@ class Base_Page(Borg, unittest.TestCase):
             element = self.get_element(locator)
             element.send_keys(Keys.PAGE_DOWN)
             self.wait(wait_time)
+            return True
         except Exception as e:
             self.write(str(e), 'debug')
-            self.exceptions.append("An exception occured when scrolling down")
-            return None
+            self.exceptions.append("An exception occurred when scrolling down")
+            return False
 
     def scroll_up(self, locator, wait_time=0.5):
         "Scroll down"
@@ -1097,10 +1096,11 @@ class Base_Page(Borg, unittest.TestCase):
             element = self.get_element(locator)
             element.send_keys(Keys.PAGE_UP)
             self.wait(wait_time)
+            return True
         except Exception as e:
             self.write(str(e), 'debug')
-            self.exceptions.append("An exception occured when scrolling up")
-            return None
+            self.exceptions.append("An exception occurred when scrolling up")
+            return False
 
     def key_press_END(self, locator, wait_time=0.5):
         "Scroll down"
@@ -1108,10 +1108,11 @@ class Base_Page(Borg, unittest.TestCase):
             element = self.get_element(locator)
             element.send_keys(Keys.END)
             self.wait(wait_time)
+            return True
         except Exception as e:
             self.write(str(e), 'debug')
-            self.exceptions.append("An exception occured when scrolling down")
-            return None
+            self.exceptions.append("An exception occurred when scrolling down")
+            return False
 
     def hover(self, locator, wait_seconds=0.5):
         "Hover over the element"
@@ -1189,14 +1190,14 @@ class Base_Page(Borg, unittest.TestCase):
             self.gif_file_name = Gif_Maker.make_gif(self.screenshot_dir, name=self.calling_module)
             return self.gif_file_name
 
-    def wait(self, wait_seconds=5, locator=None):
+    def wait(self, wait_seconds=5.0, locator=None):
         "Performs wait for time provided"
         if locator is not None:
             self.smart_wait(locator, wait_seconds=wait_seconds)
         else:
             time.sleep(wait_seconds)
 
-    def smart_wait(self, locator, wait_seconds=5, verbose_flag=True):
+    def smart_wait(self, locator, wait_seconds=5.0, verbose_flag=True):
         "Performs an explicit wait for a particular dom_element. Return True if wait success"
         result_flag = False
         try:
@@ -1270,7 +1271,7 @@ class Base_Page(Borg, unittest.TestCase):
                 self.write(negative, level='error')
 
     def execute_javascript(self, js_script, *args):
-        "Execute javascipt"
+        "Execute javascript"
         try:
             self.driver.execute_script(js_script)
             result_flag = True
@@ -1289,7 +1290,7 @@ class Base_Page(Borg, unittest.TestCase):
         return result
 
     def write_test_summary(self):
-        "Print out a useful, human readable summary"
+        "Print out a useful, human-readable summary"
         self.write(
             '\n\n************************\n--------RESULT--------\nTotal number of checks=%d' % self.result_counter)
         self.write(
@@ -1321,12 +1322,12 @@ class Base_Page(Borg, unittest.TestCase):
 
     @Wrapit._screenshot
     def take_screenshot(self):
-        """Use this function to take screenshot, the screen shot will take only when screenshot option = all"""
+        """Use this function to take screenshot, the screenshot will take only when screenshot option = all"""
         return True
 
     @Wrapit._screenshot
     def take_force_screenshot(self):
-        """Use this function to take screenshot, the screen shot will take even use screenshot option = failonly"""
+        """Use this function to take screenshot, the screenshot will take even use screenshot option = failonly"""
         return False
 
     @Wrapit._check_browser_console_log
@@ -1346,7 +1347,7 @@ class Base_Page(Borg, unittest.TestCase):
             element.scroll_by_amount(delta_x, delta_y)
         except Exception as e:
             self.write(str(e), 'debug')
-            self.exceptions.append("An exception occured when scroll by amount")
+            self.exceptions.append("An exception occurred when scroll by amount")
             return None
 
     # _get_locator = staticmethod(_get_locator) "COMMENT BECAUSE _get_locator commented"
